
package de.codedo.conflicteditor;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.URL;

public class UrlConnectionHttpAccess extends Object implements HttpAccess
{
    private static final String HEADER_CONTENT_LENGTH = "Content-Length";
    private static final String HEADER_CONTENT_TYPE = "Content-Type";
    private static final String HTTP_METHOD_DELETE = "DELETE";
    private static final String HTTP_METHOD_POST = "POST";
    private static final String HTTP_METHOD_PUT = "PUT";

    private URL _url;

    public UrlConnectionHttpAccess(URL url)
    {
        super();
        _url = url;
    }

    @Override
    public InputStream post(String content) throws IOException
    {
        HttpURLConnection connection = createUrlConnection(HTTP_METHOD_POST, content);
        send(connection, content);
        return connection.getInputStream();
    }

    @Override
    public InputStream put(String content) throws IOException
    {
        HttpURLConnection connection = createUrlConnection(HTTP_METHOD_PUT, content);
        send(connection, content);
        return connection.getInputStream();
    }

    @Override
    public InputStream delete() throws IOException
    {
        HttpURLConnection connection = createUrlConnection(HTTP_METHOD_DELETE);
        connection.connect();
        return connection.getInputStream();
    }

    private HttpURLConnection createUrlConnection(String method, String content) throws IOException, ProtocolException
    {
        HttpURLConnection connection = createUrlConnection(method);
        connection.setDoInput(true);
        connection.setRequestProperty(HEADER_CONTENT_TYPE, "application/json");
        connection.setRequestProperty(HEADER_CONTENT_LENGTH, Integer.toString(content.length()));
        return connection;
    }

    private HttpURLConnection createUrlConnection(String method) throws IOException
    {
        HttpURLConnection connection = (HttpURLConnection)_url.openConnection();
        connection.setRequestMethod(method);
        connection.setDoOutput(true);
        return connection;
    }

    private void send(HttpURLConnection connection, String content) throws IOException
    {
        DataOutputStream output = new DataOutputStream(connection.getOutputStream());
        output.writeBytes(content);
        output.flush();
        output.close();
    }

    @Override
    public InputStream get() throws IOException
    {
        HttpURLConnection connection = (HttpURLConnection)_url.openConnection();
        return connection.getInputStream();
    }
}
